@@ -263,12 +263,9 @@ bool ImportCertificatesCommand::Private::showPleaseCertify(const GpgME::Import &
         qCDebug(KLEOPATRA_LOG) << q << __func__ << "Secret key is available -> skipping certification";
         return false;
     }
-
-    for (const auto &uid : key.userIDs()) {
-        if (uid.validity() >= GpgME::UserID::Marginal) {
-            // Already marginal so don't bug the user
-            return false;
-        }
+    if (Kleo::maximalValidityOfUserIDs(key) >= GpgME::UserID::Marginal) {
+        // key has at least marginal validity so don't bug the user
+        return false;
     }
 
     const QStringList suggestions = {
@@ -810,9 +807,7 @@ void ImportCertificatesCommand::Private::startImport(GpgME::Protocol protocol,
     };
 
     job->setImportFilter(options.importFilter);
-#if QGPGME_IMPORT_JOB_SUPPORTS_IMPORT_OPTIONS
     job->setImportOptions(options.importOptions);
-#endif
     job->setKeyOrigin(options.keyOrigin, options.keyOriginUrl);
     const GpgME::Error err = job->startLater(data);
     if (err.code()) {
