@@ -40,6 +40,7 @@
 #include "commands/dumpcrlcachecommand.h"
 #include "commands/exportpaperkeycommand.h"
 #include "commands/exportsecretkeycommand.h"
+#include "commands/exportsecretteamkeycommand.h"
 #include "commands/importcertificatefromfilecommand.h"
 #include "commands/importcrlcommand.h"
 #include "commands/lookupcertificatescommand.h"
@@ -372,6 +373,15 @@ void KeyListController::createActions(KActionCollection *coll)
             nullptr,
             QString(),
         },
+        {
+            "file_export_secret_team_key",
+            i18nc("@action:inmenu", "Save Secret Team Key..."),
+            i18nc("@info:tooltip", "Save the selected secret team key to share it with other team members"),
+            "view-certificate-export-secret",
+            nullptr,
+            nullptr,
+            QString(),
+        },
         {
             "file_export_paper_key",
             i18n("Print Secret Key..."),
@@ -557,7 +567,7 @@ void KeyListController::createActions(KActionCollection *coll)
         },
         // View menu
         {
-            "redisplay",
+            "reload_certificate_list",
             i18n("Redisplay"),
             QString(),
             "view-refresh",
@@ -694,6 +704,7 @@ void KeyListController::createActions(KActionCollection *coll)
     //---
     registerActionForCommand<ExportCertificateCommand>(coll->action(QStringLiteral("file_export_certificates")));
     registerActionForCommand<ExportSecretKeyCommand>(coll->action(QStringLiteral("file_export_secret_keys")));
+    registerActionForCommand<ExportSecretTeamKeyCommand>(coll->action(QStringLiteral("file_export_secret_team_key")));
     registerActionForCommand<ExportPaperKeyCommand>(coll->action(QStringLiteral("file_export_paper_key")));
     registerActionForCommand<ExportOpenPGPCertsToServerCommand>(coll->action(QStringLiteral("file_export_certificates_to_server")));
 #ifdef MAILAKONADI_ENABLED
@@ -708,7 +719,7 @@ void KeyListController::createActions(KActionCollection *coll)
     registerActionForCommand<ChecksumCreateFilesCommand>(coll->action(QStringLiteral("file_checksum_create_files")));
     registerActionForCommand<ChecksumVerifyFilesCommand>(coll->action(QStringLiteral("file_checksum_verify_files")));
 
-    registerActionForCommand<ReloadKeysCommand>(coll->action(QStringLiteral("view_redisplay")));
+    registerActionForCommand<ReloadKeysCommand>(coll->action(QStringLiteral("reload_certificate_list")));
     // coll->action( "view_stop_operations" ) <-- already dealt with in make_actions_from_data()
     registerActionForCommand<DetailsCommand>(coll->action(QStringLiteral("view_certificate_details")));
 
@@ -1042,6 +1053,14 @@ Command::Restrictions KeyListController::Private::calculateRestrictionsMask(cons
         }
     }
 
+    if (std::ranges::all_of(keys, [](const auto &key) {
+            return !key.isBad() && std::ranges::any_of(key.subkeys(), [](const auto &subkey) {
+                return subkey.canEncrypt() && !subkey.isBad() && subkey.isSecret();
+            });
+        })) {
+        result |= Command::NeedSecretEncryptSubkey;
+    }
+
     return result;
 }
 
